// ST60 LOG SPEED SIMULATOR
// Copyright December 2015 by Nigel Mercier www.mercier.org.uk
// PIC12F1572 with PWM module
// mikroElektronica Libraries used: UART

#include "typedefs.h"         // typedef unsigned short int byte;
#include "ST60LogEmFunctions.h"
#include "NMEAfn.h"

// Global variables
sbit LED at  RA2_bit ;         // indicate valid RMC (short flash), or error (long flashes)
sbit nCAL at RA0_bit ;         // calibration mode if bit 0 held low by jumper

/*------------ Port Allocation ----------
RA0 ICSPDAT > PGD  (unset TXCKSEL b2 in APFCON for dummy TX on RA0) OK
RA1 ICSPCLK > PGC
RA2 DO LED2
RA3 ISSP MCLR
RA4 DO ST60 PWM DRIVE (set P2SEL b1 in APFCON)                          OK
RA5 DI RX NMEA IN (set RXDTSEL b7 in APFCON for RX on RA5)          OK
-----------------------------------------*/

void main( void )
{
  // local variables
  byte i, in ;
  volatile unsigned int speedE2 ;     // 100* received speed

  // SFR setup
  // bit plan  0b76543210
  OSCCON =     0b01111010 ;// set 16MHz clock IRCFx & internal osc SCSx = 0bx1110x1x
  TRISA =      0b00100001 ;// PORT A In: bit 5 (NMEA), bit 0 calibration jumper, all others OP
  ANSELA =     0b00000000 ;// select all inputs as digital
  WPUA =       0b00000001 ;// enable weak pull-ups Port A b0
  nWPUEN_bit = 0 ;         // enable WPUA weak pull-ups
  C1ON_bit = 0 ;           // Disable comparators
  RXDTSEL_bit = 1 ;        // redirect port RX: APFCON b7 RXDTSEL: RX -> RA5    (DS P102)
  SLRCONA =    0b00110111 ;// RA4 slew rate limited
  configPWM() ;            // configure PWM module and PWM output
  initNMEA() ;             // initialise NMEA port
  // main loop
  while(1)
  {
    // get speed into string speed[] from NMEA or cal mode
    time_Out = 65535 ;     // set timer for no data
    if (nCAL)
    { // not calibration mode so get speed from NMEA
      in = parseNMEA() ;   // returns 0 if parsed OK, string speed[] can be used if no timeout
      if (!time_Out)       // if time_Out = 0 then has timed out
        in = 1 ;           // timed out so do one long error flash
    }
    else
    { // calibration mode
    in = 0 ;               // flag no errors
    copySpeed("5.00") ;    // 5 knots = 27.5Hz
    }
    if (!in)             // in == 0 for no errors (so !n is true), otherwise number of errors
    { // do this if no errors (!n is true)
      LED = 1 ;                      // quick flash LED on to show zero errors
      speedE2 = strToInt(speed) ;    // convert string speed to integer times 100
      if (speedE2 < 5)               // < 0.05kn = stopped, so avoid div/0 or register overflow
      { PWM2PR = 65535 ; }           // 65535 gives very long waveform period, will show 0 knots
      else                           // speed not zero, use for setting PWM2 period
      // PWM2PR = 526 gives 5.00 knots (cal speed) but 530 gives 27.5Hz
      { PWM2PR = round_div((500*530), speedE2) ; }  // rounded div /speedE2 for PWM2 period
      PWM2LDA_A_bit = 1 ;            // reload buffers on next event
      LED = 0 ;                      // LED off
    }
    else // error count in > 0 (!n is false)
    { // errors found, long flash LED for each error
      if (!time_Out)
      { // no NMEA data
        PWM2PR = 65535 ;    // set speed as zero
        PWM2LDA_A_bit = 1 ;
      }
      for (i = 0; i < in ; i++)
      {
        LED = 1 ;        // long flash LED on
        delayX25ms(1) ;  // 25mS on
        LED = 0 ;
        delayX25ms(10) ; // 250ms LED off pause
      }
    }
  }
}